from visions.application.summaries.series.base_summary import base_summary
from visions.application.summaries.series.category_summary import category_summary
from visions.application.summaries.series.email_address_summary import (
    email_address_summary,
)
from visions.application.summaries.series.file_summary import file_summary
from visions.application.summaries.series.image_summary import image_summary
from visions.application.summaries.series.missing_summary import missing_summary
from visions.application.summaries.series.numerical_basic_summary import (
    numerical_basic_summary,
)
from visions.application.summaries.series.numerical_summary import numerical_summary
from visions.application.summaries.series.path_summary import path_summary
from visions.application.summaries.series.range_summary import range_summary
from visions.application.summaries.series.text_summary import text_summary
from visions.application.summaries.series.unique_summary import (
    unique_summary,
    unique_summary_complex,
)
from visions.application.summaries.series.url_summary import url_summary
from visions.application.summaries.series.zero_summary import zero_summary
